import java.util.*;

public class Edge
{
	public String _substrate;
	public String _product;
	
	public HashMap<String, HashSet<String>> _enzyme_reaction_set_map = new HashMap<String, HashSet<String>>();
	
	public Edge(String substrate, String product)
	{
		this._substrate = substrate;
		this._product = product;
	}
	
	public void Add_Reaction(KGMLReaction kgml_reaction)
	{
		Iterator<Map.Entry<String, HashSet<String>>> enzyme_reaction_set_map_iterator = kgml_reaction._enzyme_reaction_set_map.entrySet().iterator();
		while (enzyme_reaction_set_map_iterator.hasNext())
		{
			Map.Entry<String, HashSet<String>> current_enzyme_reaction_set_map = enzyme_reaction_set_map_iterator.next();
			String current_enzyme = current_enzyme_reaction_set_map.getKey();
			HashSet<String> current_reaction_set = current_enzyme_reaction_set_map.getValue();
			
			if (this._enzyme_reaction_set_map.containsKey(current_enzyme) == false)
				this._enzyme_reaction_set_map.put(current_enzyme, new HashSet<String>());
			
			HashSet<String> edge_enzyme_reaction_set = this._enzyme_reaction_set_map.get(current_enzyme);
			edge_enzyme_reaction_set.addAll(current_reaction_set);
		}
	}
	
	public String Output_Line()
	{
		if (Parameters._Graph_Level == Parameters._GL_COMPOUND)
		{
			return this.Output_Line_Compound_Level();
		}
		else if (Parameters._Graph_Level == Parameters._GL_ENZYME)
		{
			return this.Output_Line_Enzyme_Level();
		}
		else if (Parameters._Graph_Level == Parameters._GL_REACTION)
		{
			return this.Output_Line_Reaction_Level();
		}
		else if (Parameters._Graph_Level == Parameters._GL_REACTION_ENZYME)
		{
			return this.Output_Line_Reaction_Enzyme_Level();
		}

		return null;
	}
	
	public String Output_Line_Compound_Level()
	{
		String output_string = new String();
		
		output_string += this._substrate + "," + this._product + "\t";
		output_string += this._substrate + "\t";
		output_string += this._product + "\t";
		
		Iterator<String> enzyme_iterator = this._enzyme_reaction_set_map.keySet().iterator();
		while (enzyme_iterator.hasNext())
		{
			output_string += enzyme_iterator.next();
			
			if (enzyme_iterator.hasNext())
				output_string += ",";
		}
		
		output_string += "\n";
		
		return output_string;
	}
	
	public String Output_Line_Enzyme_Level()
	{
		String output_string = new String();
		
		Iterator<String> enzyme_iterator = this._enzyme_reaction_set_map.keySet().iterator();
		while (enzyme_iterator.hasNext())
		{
			String current_enzyme = enzyme_iterator.next();
			
			output_string += this._substrate + "," + this._product + "," + current_enzyme + "\t";
			output_string += this._substrate + "\t";
			output_string += this._product + "\t";
			output_string += current_enzyme;
			
			output_string += "\n";
		}
		
		return output_string;
	}
	
	public String Output_Line_Reaction_Level()
	{
		String output_string = new String();
		
		HashMap<String, HashSet<String>> reaction_enzyme_set_map = new HashMap<String, HashSet<String>>();
		
		Iterator<Map.Entry<String, HashSet<String>>> enzyme_reaction_set_map_iterator = this._enzyme_reaction_set_map.entrySet().iterator();
		while (enzyme_reaction_set_map_iterator.hasNext())
		{
			Map.Entry<String, HashSet<String>> current_enzyme_reaction_set_map = enzyme_reaction_set_map_iterator.next();
			String current_enzyme = current_enzyme_reaction_set_map.getKey();
			HashSet<String> current_reaction_set = current_enzyme_reaction_set_map.getValue();

			Iterator<String> reaction_iterator = current_reaction_set.iterator();
			while (reaction_iterator.hasNext())
			{
				String current_reaction = reaction_iterator.next();
				
				if (!reaction_enzyme_set_map.containsKey(current_reaction))
					reaction_enzyme_set_map.put(current_reaction, new HashSet<String>());
				
				HashSet<String> reaction_enzymes = reaction_enzyme_set_map.get(current_reaction);
				reaction_enzymes.add(current_enzyme);
			}
		}
		
		Iterator<Map.Entry<String, HashSet<String>>> reaction_enzyme_set_map_iterator = reaction_enzyme_set_map.entrySet().iterator();
		while (reaction_enzyme_set_map_iterator.hasNext())
		{
			Map.Entry<String, HashSet<String>> current_reaction_enzyme_set_map = reaction_enzyme_set_map_iterator.next();
			String current_reaction_name = current_reaction_enzyme_set_map.getKey();
			HashSet<String> enzyme_set = current_reaction_enzyme_set_map.getValue();
			
			output_string += this._substrate + "," + this._product + "," + current_reaction_name + "\t";
			output_string += this._substrate + "\t";
			output_string += this._product + "\t";

			Iterator<String> reaction_enzyme_iterator = enzyme_set.iterator();
			while (reaction_enzyme_iterator.hasNext())
			{
				output_string += reaction_enzyme_iterator.next();
				
				if (reaction_enzyme_iterator.hasNext())
					output_string += ",";
			}
			
			output_string += "\n";
		}
		
		return output_string;
	}
	
	public String Output_Line_Reaction_Enzyme_Level()
	{
		String output_string = new String();
		
		Iterator<Map.Entry<String, HashSet<String>>> enzyme_reaction_set_map_iterator = this._enzyme_reaction_set_map.entrySet().iterator();
		while (enzyme_reaction_set_map_iterator.hasNext())
		{
			Map.Entry<String, HashSet<String>> current_enzyme_reaction_set_map = enzyme_reaction_set_map_iterator.next();
			String current_enzyme = current_enzyme_reaction_set_map.getKey();
			HashSet<String> current_reaction_set = current_enzyme_reaction_set_map.getValue();

			Iterator<String> reaction_iterator = current_reaction_set.iterator();
			while (reaction_iterator.hasNext())
			{
				String current_reaction = reaction_iterator.next();

				output_string += this._substrate + "," + this._product + "," + current_enzyme + "," + current_reaction + "\t";
				output_string += this._substrate + "\t";
				output_string += this._product + "\t";
				output_string += current_enzyme;
				
				output_string += "\n";
			}
		}
		
		return output_string;
	}
}
