import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;

public class KGMLData 
{
	public static HashMap<String, KGMLReaction> _KGML_Reactions = new HashMap<String, KGMLReaction>();
	public static HashMap<String, String> _Compounds = new HashMap<String, String>();
	
	public static void Read()
	{
		try
		{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			
			Document xml_document = builder.parse(new File(Parameters._KGML_File_Name));
			Element document_element = xml_document.getDocumentElement();
			
			Parse_Entry_Data(document_element.getElementsByTagName("entry"));
			Parse_Reaction_Data(document_element.getElementsByTagName("reaction"));	
		}
		catch (Throwable t)
		{
			System.err.println("ERROR: Reading KGML data");
			System.exit(1);
		}
	}
	
	public static void Parse_Entry_Data(NodeList entry_list)
	{
		for (int i = 0; i < entry_list.getLength(); i++)
		{
			Element current_entry = (Element) entry_list.item(i);
			String id = current_entry.getAttribute("id");
			
			if (current_entry.getAttribute("type").compareTo("enzyme") == 0)
			{
				KGMLReaction new_kgml_reaction = new KGMLReaction();
				
				String enzyme_string = current_entry.getAttribute("name");
				String reaction_string = current_entry.getAttribute("reaction");
				
				String[] enzyme_names = enzyme_string.split(" ");
				String[] reaction_names = reaction_string.split(" ");
				
				new_kgml_reaction.Add_Enzyme_Reaction_Mappings(enzyme_names, reaction_names);
				
				KGMLData._KGML_Reactions.put(id, new_kgml_reaction);
			}
			else if (current_entry.getAttribute("type").compareTo("compound") == 0)
			{
				String compound_name = current_entry.getAttribute("name");
				KGMLData._Compounds.put(id, compound_name + "-" + id);
			}
		}
	}
	
	public static void Parse_Reaction_Data(NodeList reaction_list)
	{
		for (int i = 0; i < reaction_list.getLength(); i++)
		{
			Element current_reaction = (Element) reaction_list.item(i);
			String id = current_reaction.getAttribute("id");
			
			if (KGMLData._KGML_Reactions.containsKey(id) == false)
				KGMLData._KGML_Reactions.put(id, new KGMLReaction());
			
			KGMLReaction current_kgml_reaction = KGMLData._KGML_Reactions.get(id);
			
			NodeList substrate_list = current_reaction.getElementsByTagName("substrate");
			NodeList product_list = current_reaction.getElementsByTagName("product");
			
			for (int j = 0; j < substrate_list.getLength(); j++)
			{
				Element current_substrate = (Element) substrate_list.item(j);
				String substrate_id = current_substrate.getAttribute("id");
				
				current_kgml_reaction._substrates.add(KGMLData._Compounds.get(substrate_id));
			}

			for (int j = 0; j < product_list.getLength(); j++)
			{
				Element current_product = (Element) product_list.item(j);
				String product_id = current_product.getAttribute("id");
				
				current_kgml_reaction._products.add(KGMLData._Compounds.get(product_id));
			}
		}
	}
}
