import java.io.File;


public class Parameters
{
	public static final int _GL_COMPOUND = 1;
	public static final int _GL_ENZYME = 2;
	public static final int _GL_REACTION = 3;
	public static final int _GL_REACTION_ENZYME = 4;
	
	public static final int _DEFAULT_GRAPH_LEVEL = _GL_COMPOUND;
	
	
	public static String _KGML_File_Name = null;
	public static String _Output_Graph_File_Name = null;
	
	public static String _Graph_Level_Input_String = null;
	public static int _Graph_Level = _GL_ENZYME;
	
	
	public static void Set_Parameters(String[] args)
	{
		if (args.length == 0)
		{
			Print_Usage();
			System.exit(0);
		}
		
		int parameter_index = 0;
		
		while (parameter_index < args.length)
			parameter_index = Parse_Arguments(args, parameter_index);
		
		Check_Requirements();
		Finalize_Parameters();
	}
	
	public static void Print_Usage()
	{
		String output_string = new String();
		
		output_string += "Input Parameters for the KGMLDataParser algorithm:\n";
		output_string += "  <-k>  REQUIRED : Name of KGML File\n";
		output_string += "  <-o>  REQUIRED : Name of file to containing the output graph\n";
		output_string += "  <-l>  DEFAULT=COMPOUND : Graph specificity level (COMPOUND, ENZYME, REACTION, or REACTION_ENZYME)\n";
		
		System.err.print(output_string);
	}

	
	public static int Parse_Arguments(String[] args, int index)
	{
		if (args[index].compareTo("-k") == 0)
		{
			index++;
			Parameters._KGML_File_Name = args[index++];
		}
		else if (args[index].compareTo("-o") == 0)
		{
			index++;
			Parameters._Output_Graph_File_Name = args[index++];
		}
		else if (args[index].compareTo("-l") == 0)
		{
			index++;
			Parameters._Graph_Level_Input_String = args[index++]; 
		}
		else
		{
			System.err.println("WARNING: Parameter \"" + args[index++] + "\" ignored");
		}
		
		return index;
	}
	
	public static void Check_Requirements()
	{
		if (Parameters._KGML_File_Name == null ||
				Parameters._Output_Graph_File_Name == null)
		{
			System.err.println("ERROR: One or more required arguments not provided");
			Print_Usage();
			System.exit(1);			
		}
		
		boolean error_flag = false;
		
		File kgml_file = new File(Parameters._KGML_File_Name);
		
		if (kgml_file.isFile() == false)
		{
			System.err.println("ERROR: \"" + Parameters._KGML_File_Name + "\" is not a file");
			error_flag = true;
		}
		
		if (error_flag)
		{
			System.exit(1);
		}
	}
	
	public static void Finalize_Parameters()
	{
		if (Parameters._Graph_Level_Input_String == null)
		{
			Parameters._Graph_Level = Parameters._DEFAULT_GRAPH_LEVEL;
		}
		else
		{
			if (Parameters._Graph_Level_Input_String.compareTo("COMPOUND") == 0)
				Parameters._Graph_Level = Parameters._GL_COMPOUND;
			else if (Parameters._Graph_Level_Input_String.compareTo("ENZYME") == 0)
				Parameters._Graph_Level = Parameters._GL_ENZYME;
			else if (Parameters._Graph_Level_Input_String.compareTo("REACTION") == 0)
				Parameters._Graph_Level = Parameters._GL_REACTION;
			else if (Parameters._Graph_Level_Input_String.compareTo("REACTION_ENZYME") == 0)
				Parameters._Graph_Level = Parameters._GL_REACTION_ENZYME;
			else
			{
				System.err.println("WARNING: Graph level argument unrecognized.  Using default");
				Parameters._Graph_Level = Parameters._DEFAULT_GRAPH_LEVEL;
			}
		}
	}
}
