
public class AlgorithmParameters
{
	public static final int SCREEN = 1;
	public static final int FILE = 2;
	
	public static final int GLOBAL = 3;
	public static final int LOCAL = 4;
	
	public String _network_map_filename;
	
	public String _instance_list_filename;
	
	public String _instance_directory_name;
	public String _instance_file_extension = ".eclist";

	public int _output_destination = SCREEN;
	public String _output_filename = "";
	
	public int _max_seed_size = 4;	
	public double _bias_threshold = 0.5;
	public double _degredation_bound = 1.0;
	
	public int _seed_scope = GLOBAL;

	public AlgorithmParameters(String[] args)
	{
		if (args.length == 0)
		{
			this.Print_Usage();
			System.exit(0);
		}
		
		int param_index = 0;
		
		while (param_index < args.length)
		{
			param_index = Parse_Param(param_index, args);
			
			if (param_index < 0)
				System.exit(1);
		}
		
	/*
		for (int i = 0; i < args.length; i += 2)
		{
			String param_type = args[i];
			String param_value = args[i+1];
			
			this.Parse_Param(param_type, param_value);
		}
	*/
	}
	
	private int Parse_Param(int param_index, String[] args)
	{
		String param_type = args[param_index++];
		
		if (param_type.compareTo("-m") == 0)
		{
			this._network_map_filename = args[param_index++];
			return param_index;
		}
		
		if (param_type.compareTo("-o") == 0)
		{
			this._output_destination = FILE;
			this._output_filename = args[param_index++];
			return param_index;
		}
		
		if (param_type.compareTo("-l") == 0)
		{
			this._instance_list_filename = args[param_index++];
			return param_index;
		}
		
		if (param_type.compareTo("-d") == 0)
		{
			this._instance_directory_name = args[param_index++];

			if (this._instance_directory_name.endsWith("/") == false)
				this._instance_directory_name += "/";
			
			return param_index;
		}
		
		if (param_type.compareTo("-e") == 0)
		{
			this._instance_file_extension = args[param_index++];
			return param_index;
		}
		
		if (param_type.compareTo("-s") == 0)
		{
			this._max_seed_size = Integer.parseInt(args[param_index++]);
			return param_index;
		}
		
		if (param_type.compareTo("-b") == 0)
		{
			double input_value = Double.parseDouble(args[param_index++]);
			
			if (input_value <= 1.0 && input_value > 0)
				this._bias_threshold = Double.parseDouble(args[param_index++]);
			else
				System.out.println("ERROR: Input value for parameter '-b' is invalid.  Using default value (0.5)");

			return param_index;
		}
		
		if (param_type.compareTo("-x") == 0)
		{
			double input_value = Double.parseDouble(args[param_index++]);
			
			if (input_value <= 1.0 && input_value > 0)
				this._degredation_bound = input_value;
			else
				System.out.println("ERROR: Input value for parameter '-x' is invalid.  Using default value (1.0)");
			
			return param_index;
		}
		
		if (param_type.compareTo("-v") == 0)
		{
			String input_value = args[param_index++];
			
			if (input_value.compareToIgnoreCase("G") == 0)
				this._seed_scope = GLOBAL;
			else if (input_value.compareToIgnoreCase("L") == 0)
				this._seed_scope = LOCAL;
			else
				System.err.println("ERROR: Input value for parameter '-v' is invalid. Using default value ('G')");
			
			return param_index;
		}
		
		this.Print_Usage();
		return -1;
	}
	
	private void Print_Usage()
	{
		System.out.println("Input parameters for the NIBBS-Search Algorithm:");
		System.out.println("  <-m> : REQUIRED : Filename of Network Map");
		System.out.println("  <-l> : REQUIRED : Filename of the Instance List with Types (P or N)");
		System.out.println("  <-d> : REQUIRED : Name of Directory Containing Instance Edge Tags");
		System.out.println("  <-o> : OPTIONAL : DEFAULT = stdout      : Filename of the Output File");
		System.out.println("  <-e> : OPTIONAL : DEFAULT = \".eclist\"   : File Extension of Instance Edge Tag Files");
		System.out.println("  <-b> : OPTIONAL : DEFAULT = 0.5         : Bias Score Threshold Value for Seed Expansion");
		System.out.println("  <-s> : OPTIONAL : DEFAULT = 4           : Maximum Number of Edges in a Seed");
		System.out.println("  <-x> : OPTIONAL : DEFAULT = 1.0         : Minimum Percentage of Positive Instances Retained During Seed Expansion");
		System.out.println("  <-v> : OPTIONAL : DEFAULT = 'G'         : Scope of the Seed Expansion; Global (G) or Local (L)");
	}
}
