import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.*;

public class ConservationProfile
{
	public static double [][] _BIAS_VALUES;
	public static double _POPULATION_SIZE;
	public static double _TOTAL_NUMBER_OF_POSITIVE_INSTANCES;
	
	public HashSet<NetworkInstance> _positive_instances;
	public HashSet<NetworkInstance> _negative_instances;
	
	public ConservationProfile(Edge edge, ConservationProfile conservation_profile)
	{
		this._positive_instances = new HashSet<NetworkInstance>();
		this._negative_instances = new HashSet<NetworkInstance>();
		
		Iterator<NetworkInstance> edge_instance_iterator = edge._instances.iterator();
		while (edge_instance_iterator.hasNext())
		{
			NetworkInstance current_instance = edge_instance_iterator.next();
			if (conservation_profile._positive_instances.contains(current_instance))
				this._positive_instances.add(current_instance);
			if (conservation_profile._negative_instances.contains(current_instance))
				this._negative_instances.add(current_instance);
		}
	}
	
	public ConservationProfile(String file_name)
	{
		this._positive_instances = new HashSet<NetworkInstance>();
		this._negative_instances = new HashSet<NetworkInstance>();
		
		try
		{
			FileInputStream il_fis = new FileInputStream(file_name);
			InputStreamReader il_isr = new InputStreamReader(il_fis);
			BufferedReader il_reader = new BufferedReader(il_isr);
			
			String read_line;
			
			while ((read_line = il_reader.readLine()) != null)
			{
				String [] line_tokens = read_line.split("\t");
				
				String instance_type = line_tokens[0];
				String instance_name = line_tokens[1];
				
				if (instance_type.compareTo("P") == 0)
					this._positive_instances.add(new NetworkInstance(instance_name));
				else if(instance_type.compareTo("N") == 0)
					this._negative_instances.add(new NetworkInstance(instance_name));
			}
			
			il_reader.close();
			il_isr.close();
			il_fis.close();
		}
		catch(Throwable t)
		{
			System.out.println("Exiting");
			System.exit(100);
		}
		
		ConservationProfile._POPULATION_SIZE = this._positive_instances.size() + this._negative_instances.size();
		ConservationProfile._TOTAL_NUMBER_OF_POSITIVE_INSTANCES = this._positive_instances.size();
		
		int array_rows = (int) ConservationProfile._POPULATION_SIZE + 1;
		int array_cols = (int) ConservationProfile._TOTAL_NUMBER_OF_POSITIVE_INSTANCES + 1;
		
		ConservationProfile._BIAS_VALUES = new double[array_rows][array_cols];
	}
	
	public double Calculuate_Bias_Score()
	{		
		int row = this._positive_instances.size() + this._negative_instances.size();
		int col = this._positive_instances.size();
		
		if (ConservationProfile._BIAS_VALUES[row][col] == 0)
			Calculate_Bias_Value(row, col);
		
		return ConservationProfile._BIAS_VALUES[row][col];
	}
	
	public void Calculate_Bias_Value(int sample_size, int sample_successes)
	{	
		double hypergeometric_probability = Calculate_Hypergeometric_Probability(sample_size, sample_successes);
		double cumulative_hypergeometric_probability = hypergeometric_probability;
		
		if (sample_successes < sample_size &&
			sample_successes < ConservationProfile._TOTAL_NUMBER_OF_POSITIVE_INSTANCES)
		{
			if (ConservationProfile._BIAS_VALUES[sample_size][sample_successes + 1] == 0)
				Calculate_Bias_Value(sample_size, sample_successes + 1);
			
			cumulative_hypergeometric_probability += ConservationProfile._BIAS_VALUES[sample_size][sample_successes + 1];
		}

		ConservationProfile._BIAS_VALUES[sample_size][sample_successes] = cumulative_hypergeometric_probability;		
	}
	
	private double Calculate_Hypergeometric_Probability(double sample_size, double sample_successes)
	{
		double population_size = ConservationProfile._POPULATION_SIZE;
		double population_successes = ConservationProfile._TOTAL_NUMBER_OF_POSITIVE_INSTANCES;
		
		double numerator_1 = Calculate_Binomial_Coefficient(population_successes, sample_successes);
		double numerator_2 = Calculate_Binomial_Coefficient(population_size - population_successes, sample_size - sample_successes);
		
		double denominator = Calculate_Binomial_Coefficient(population_size, sample_size);
		
		return (numerator_1 * numerator_2) / denominator;		
	}
	
	private double Calculate_Binomial_Coefficient(double n, double r)
	{
		if ((n-r) < r)
			r = n-r;
		
		double coefficient = 1.0;
		
		for (int i = 0; i < r; i++)
		{
			coefficient *= (n-i);
			coefficient /= (r-i);
		}
		
		return coefficient;
	}
}
