import java.util.*;

public class Edge
{
	public static final int EXPECTED_NUMBER_OF_EDGE_TOKENS = 4;
	public static final int EXPECTED_NUMBER_OF_ENDPOINTS = 2;

	public String _edge_name;
	
	public Vertex _endpoint_1;
	public Vertex _endpoint_2;
	
	public HashSet<Tag> _tags;
	public HashSet<NetworkInstance> _instances;
	public HashSet<Edge> _adjacent_edges;
	
	public boolean _seeded;

	public Edge(String file_string, NetworkMap network_map)
	{
		String[] edge_strings = file_string.split("\t", -1);
		
		if (edge_strings.length != EXPECTED_NUMBER_OF_EDGE_TOKENS)
			System.exit(100);
		
		this._edge_name = edge_strings[0];
		
		String endpoint_name_1 = edge_strings[1];
		String endpoint_name_2 = edge_strings[2];
		
		String tags_string = edge_strings[3];
				
		if (!network_map._vertices.containsKey(endpoint_name_1))
			network_map._vertices.put(endpoint_name_1, new Vertex(endpoint_name_1));
		
		if (!network_map._vertices.containsKey(endpoint_name_2))
			network_map._vertices.put(endpoint_name_2, new Vertex(endpoint_name_2));
		
		this._endpoint_1 = network_map._vertices.get(endpoint_name_1);
		this._endpoint_2 = network_map._vertices.get(endpoint_name_2);
		
		this._endpoint_1.Add_Edge(this._endpoint_2, this);
		this._endpoint_2.Add_Edge(this._endpoint_1, this);
				
		this._tags = new HashSet<Tag>();
		StringTokenizer tag_tokens = new StringTokenizer(tags_string, ",");
		
		while (tag_tokens.hasMoreTokens())
		{
			String tag_name = tag_tokens.nextToken();
			
			if (!network_map._tags.containsKey(tag_name))
				network_map._tags.put(tag_name, new Tag(tag_name));
			
			Tag edge_tag = network_map._tags.get(tag_name);
			this._tags.add(edge_tag);
			
			edge_tag._edges.add(this);
		}
		
		this._instances = new HashSet<NetworkInstance>();
		this._adjacent_edges = new HashSet<Edge>();
		
		this._seeded = false;
	}

	public void Find_Adjacent_Edges()
	{
		Enumeration<LinkedList<Edge>> edge_list_enumeration = this._endpoint_1._edges.elements();
		while (edge_list_enumeration.hasMoreElements())
		{
			Iterator<Edge> edge_iterator = edge_list_enumeration.nextElement().iterator();
			while (edge_iterator.hasNext())
			{
				this._adjacent_edges.add(edge_iterator.next());
			}
		}
		
		edge_list_enumeration = this._endpoint_2._edges.elements();
		while (edge_list_enumeration.hasMoreElements())
		{
			Iterator<Edge> edge_iterator = edge_list_enumeration.nextElement().iterator();
			while (edge_iterator.hasNext())
			{
				this._adjacent_edges.add(edge_iterator.next());
			}
		}
		
		this._adjacent_edges.remove(this);
	}
	
	public String toString()
	{
		String return_string = "";
		
		return_string += this._edge_name + "\t" + this._endpoint_1._label + "\t" + this._endpoint_2._label + "\t";
		
		Iterator<Tag> tag_iterator = this._tags.iterator();
		while (tag_iterator.hasNext())
		{
			return_string += tag_iterator.next()._label;

			if (tag_iterator.hasNext())
				return_string += ",";
		}
	
		return_string += "\n";
		return return_string;
	}
}
