import java.util.*;

public class EdgeSet implements Comparable<EdgeSet>
{
	public Edge _start_edge;

	public LinkedList<String> _seed_edge_names;
	public LinkedList<String> _expansion_edge_names;
	
	public HashSet<Edge> _edge_set;
	public ConservationProfile _profile;
	
	public EdgeSet (Edge start_edge, ConservationProfile network_instance_profile)
	{
		this._seed_edge_names = new LinkedList<String>();
		this._expansion_edge_names = new LinkedList<String>();
		
		this._edge_set = new HashSet<Edge>();
		this._profile = new ConservationProfile(start_edge, network_instance_profile);
		
		this._start_edge = start_edge;
		this._edge_set.add(start_edge);
	}
	
	public int compareTo(EdgeSet compare_edge_set)
	{
		double compare_bias = compare_edge_set._profile.Calculuate_Bias_Score();
		double this_bias = this._profile.Calculuate_Bias_Score();
		
		if (this_bias < compare_bias)
			return 1;
		else if (this_bias > compare_bias)
			return -1;
		else
			return 0;
	}
	
	public boolean Generate_Seed_Local()
	{
		if (this._start_edge._seeded)
			return false;
		else
			this._start_edge._seeded = true;
		
		HashSet<Edge> candidate_edges = new HashSet<Edge>();
		candidate_edges = this._start_edge._adjacent_edges;
		
		for (int i = 1; i < NIBBSSearch.params._max_seed_size; i++)
		{
			Edge current_best_candidate = null;
			ConservationProfile current_best_profile = null;
			double current_best_bias = this._profile.Calculuate_Bias_Score();
			
			Iterator<Edge> candidate_iterator = candidate_edges.iterator();
			while (candidate_iterator.hasNext())
			{
				Edge current_candidate = candidate_iterator.next();
				
				if (current_candidate._seeded)
					continue;
				
				ConservationProfile possible_profile = new ConservationProfile(current_candidate, this._profile);
				double current_bias = possible_profile.Calculuate_Bias_Score();
				
				if (current_bias < current_best_bias)
				{
					current_best_candidate = current_candidate;
					current_best_profile = possible_profile;
					current_best_bias = current_bias;
				}
			}
			
			if (current_best_candidate == null)
				break;

			current_best_candidate._seeded = true;
			this._edge_set.add(current_best_candidate);
			this._seed_edge_names.add(current_best_candidate._edge_name);
			this._profile = current_best_profile;
			
			Iterator<Edge> newly_adjacent_edge_iterator = current_best_candidate._adjacent_edges.iterator();
			while (newly_adjacent_edge_iterator.hasNext())
			{
				Edge current_edge = newly_adjacent_edge_iterator.next();
				if (this._edge_set.contains(current_edge))
					continue;
				
				candidate_edges.add(current_edge);
			}
		}
		
		return true;
	}
	
	public boolean Generate_Seed_Global(LinkedList<Edge> candidates)
	{
		if (this._start_edge._seeded)
			return false;
		else
			this._start_edge._seeded = true;
		
		for (int i = 1; i < NIBBSSearch.params._max_seed_size; i++)
		{
			Edge current_best_candidate = null;
			ConservationProfile current_best_profile = null;
			double current_best_bias = this._profile.Calculuate_Bias_Score();
			
			Iterator<Edge> candidate_iterator = candidates.iterator();
			while (candidate_iterator.hasNext())
			{
				Edge current_candidate = candidate_iterator.next();
				
				if (current_candidate._seeded)
					continue;
				
				ConservationProfile possible_profile = new ConservationProfile(current_candidate, this._profile);
				double current_bias = possible_profile.Calculuate_Bias_Score();
				
				if (current_bias < current_best_bias)
				{
					if (this.Edge_Set_Connected(current_candidate, possible_profile))
					{
						current_best_candidate = current_candidate;
						current_best_profile = possible_profile;
						current_best_bias = current_bias;
					}
				}
			}
			
			if (current_best_candidate == null)
				break;

			current_best_candidate._seeded = true;
			this._edge_set.add(current_best_candidate);
			this._seed_edge_names.add(current_best_candidate._edge_name);
			this._profile = current_best_profile;			
		}
		
		return true;
	}
	
	public boolean Edge_Set_Connected(Edge test_edge, ConservationProfile test_profile)
	{
		HashSet<Edge> visited_edges = new HashSet<Edge>();
		LinkedList<Edge> adjacent_edges = new LinkedList<Edge>();
		
		adjacent_edges.addLast(test_edge);
		
		while(adjacent_edges.size() > 0)
		{
			Edge current_edge = adjacent_edges.removeFirst();
			
			if (this._edge_set.contains(current_edge))
				return true;
			
			if (visited_edges.add(current_edge) == true)
			{
				Iterator<Edge> adjacent_edge_iterator = current_edge._adjacent_edges.iterator();
				while (adjacent_edge_iterator.hasNext())
				{
					Edge current_adjacent_edge = adjacent_edge_iterator.next();
					
					if (visited_edges.contains(current_adjacent_edge) == false)
					{
						ConservationProfile possible_profile = new ConservationProfile(current_adjacent_edge, test_profile);
						
						if (possible_profile._positive_instances.size() == test_profile._positive_instances.size())
							adjacent_edges.addLast(current_adjacent_edge);
					}
				}
			}
		}
		
		return false;
	}
	
	public void Expand_Seed()
	{
		int minimum_number_of_positive_instances = (int) Math.ceil(this._profile._positive_instances.size() * NIBBSSearch.params._degredation_bound);
		HashSet<Edge> candidate_expansion_edges = new HashSet<Edge>();
		
		Iterator<Edge> edge_iterator = this._edge_set.iterator();
		while (edge_iterator.hasNext())
		{
			Edge current_edge = edge_iterator.next();
			Iterator<Edge> adjacent_edge_enumeration = current_edge._adjacent_edges.iterator();
			while (adjacent_edge_enumeration.hasNext())
			{
				Edge current_adjacent_edge = adjacent_edge_enumeration.next();
				if (this._edge_set.contains(current_adjacent_edge))
					continue;
				
				candidate_expansion_edges.add(current_adjacent_edge);
			}
		}
		
		while (true)
		{
			Edge current_best_candidate = null;
			ConservationProfile current_best_profile = null;
			int current_best_number_of_positive_instances = -1;
			double current_best_bias = 0;
			
			Iterator<Edge> candidate_iterator = candidate_expansion_edges.iterator();
			while (candidate_iterator.hasNext())
			{
				Edge current_candidate = candidate_iterator.next();
				ConservationProfile current_profile = new ConservationProfile(current_candidate, this._profile);
				int current_number_of_positive_instances = current_profile._positive_instances.size();
				
				if (current_number_of_positive_instances < minimum_number_of_positive_instances)
				{
					candidate_iterator.remove();
					continue;
				}
				else if (current_number_of_positive_instances > current_best_number_of_positive_instances)
				{
					current_best_candidate = current_candidate;
					current_best_number_of_positive_instances = current_number_of_positive_instances;
					current_best_profile = current_profile;
					current_best_bias = current_profile.Calculuate_Bias_Score();
				}
				else if (current_number_of_positive_instances == current_best_number_of_positive_instances)
				{
					double current_bias = current_profile.Calculuate_Bias_Score();
					
					if (current_bias < current_best_bias)
					{
						current_best_candidate = current_candidate;
						current_best_number_of_positive_instances = current_number_of_positive_instances;
						current_best_profile = current_profile;
						current_best_bias = current_bias;
					}
				}
			}
			
			if (current_best_candidate == null)
				break;			

			this._edge_set.add(current_best_candidate);
			this._profile = current_best_profile;
			
			candidate_expansion_edges.remove(current_best_candidate);
			this._expansion_edge_names.add(current_best_candidate._edge_name);

			Iterator<Edge> newly_adjacent_edge_iterator = current_best_candidate._adjacent_edges.iterator();
			while (newly_adjacent_edge_iterator.hasNext())
			{
				Edge current_edge = newly_adjacent_edge_iterator.next();
				if (this._edge_set.contains(current_edge))
					continue;
				
				candidate_expansion_edges.add(current_edge);
			}
		}
	}

	public String PrintReport()
	{
		String return_string = "";
		
		return_string += "Bias Score: " + this._profile.Calculuate_Bias_Score() + "\n";

		return_string += "Positive Instances: " + this._profile._positive_instances.size();
		Iterator<NetworkInstance> positive_instance_name_iterator = this._profile._positive_instances.iterator();
		if (positive_instance_name_iterator.hasNext())
			return_string += "-";
		while (positive_instance_name_iterator.hasNext())
		{
			return_string += positive_instance_name_iterator.next()._instance_name;
			if (positive_instance_name_iterator.hasNext())
				return_string += ",";
		}
	
		return_string += "\n";
		
		return_string += "Negative Instances: " + this._profile._negative_instances.size();
		Iterator<NetworkInstance> negative_instance_name_iterator = this._profile._negative_instances.iterator();
		if (negative_instance_name_iterator.hasNext())
			return_string += "-";
		while (negative_instance_name_iterator.hasNext())
		{
			return_string += negative_instance_name_iterator.next()._instance_name;
			if (negative_instance_name_iterator.hasNext())
				return_string += ",";
		}
	
		return_string += "\n";

		return_string += "\t1:" + this._start_edge._edge_name + "\n";
		
		Iterator<String> seed_edge_name_iterator = this._seed_edge_names.iterator();
		while (seed_edge_name_iterator.hasNext())
		{
			return_string += "\ts:  " + seed_edge_name_iterator.next() + "\n";
		}
		
		Iterator<String> expansion_edge_name_iterator = this._expansion_edge_names.iterator();
		while (expansion_edge_name_iterator.hasNext())
		{
			return_string += "\tx:    " + expansion_edge_name_iterator.next() + "\n";
		}
		
		return return_string;
	}
}
