import java.io.*;
import java.util.*;

public class PathwayFinder
{
	public NetworkCollection _network_collection;
	public LinkedList<EdgeSet> _edge_sets;
	
	public PathwayFinder(NetworkCollection network_collection)
	{
		this._network_collection = network_collection;
		this._edge_sets = new LinkedList<EdgeSet>();
	}
	
	public void Find_Pathways()
	{
		Iterator<Edge> edge_iterator = this._network_collection._network_map._edges.iterator();
		while (edge_iterator.hasNext())
		{
			Edge current_edge = edge_iterator.next();
			EdgeSet new_edge_set = new EdgeSet(current_edge, this._network_collection._profile);
			
			this._edge_sets.add(new_edge_set);
		}
		
		Collections.sort(this._edge_sets);
		Collections.reverse(this._edge_sets);
		
		Iterator<EdgeSet> edge_set_iterator = this._edge_sets.iterator();
		while (edge_set_iterator.hasNext())
		{
			EdgeSet current_edge_set = edge_set_iterator.next();

			boolean generate_seed_function_response;
			
			if (NIBBSSearch.params._seed_scope == AlgorithmParameters.GLOBAL)
				generate_seed_function_response = current_edge_set.Generate_Seed_Global(this._network_collection._network_map._edges);
			else
				generate_seed_function_response = current_edge_set.Generate_Seed_Local();

			if (generate_seed_function_response == false ||
				current_edge_set._profile.Calculuate_Bias_Score() >= NIBBSSearch.params._bias_threshold)
				edge_set_iterator.remove();
		}
		
		Collections.sort(this._edge_sets);
		
		Iterator<EdgeSet> seed_set_iterator = this._edge_sets.iterator();
		while (seed_set_iterator.hasNext())
		{
			seed_set_iterator.next().Expand_Seed();
		}
		
		Collections.sort(this._edge_sets);
		Collections.reverse(this._edge_sets);
	}
	
	public void PrintReport()
	{
		if (NIBBSSearch.params._output_destination == AlgorithmParameters.FILE)
		{
			try
			{
				FileOutputStream fos = new FileOutputStream(NIBBSSearch.params._output_filename);
				OutputStreamWriter osw = new OutputStreamWriter(fos);
				BufferedWriter writer = new BufferedWriter(osw);
				
				writer.write(Report());
				
				writer.close();
				osw.close();
				fos.close();
			}
			catch(Throwable t)
			{
				System.out.println("Exiting");
				System.exit(1);
			}
		}
		else
		{
			System.out.print(Report());		
		}
	}
	
	public String Report()
	{
		String return_string = "";
		
		Iterator<EdgeSet> edge_set_iterator = this._edge_sets.iterator();
		while (edge_set_iterator.hasNext())
		{
			return_string += edge_set_iterator.next().PrintReport() + "\n";
		}
		
		return return_string;
	}
}
